package game;

import java.awt.event.KeyEvent;

import main.Input;
import main.TestGame;

public class Player extends Entity {
	
	public final int colorFlag = 0xFFFFAACC;
	public final int ENTITY_SIZE = 1;
	
	private int oldX, oldY;
	private int newX, newY;
	private double dx, dy;
	private boolean attacking;
	private int attackX, attackY;
	private boolean isSelected;
	private int step;
	private int radius = 0;

	public Player(int x, int y) {
		xPosition = x;
		yPosition = y;
		oldX = x;
		oldY = y;
		newX = newY = -1;
		isSelected = false;
		step = 0;
	}
	
	public void move(TestGame game, World world, Input input) {
		oldX = xPosition;
		oldY = yPosition;
		if (input.keys[KeyEvent.VK_UP] == true)
			yPosition--;
		if (input.keys[KeyEvent.VK_DOWN] == true)
			yPosition++;
		if (input.keys[KeyEvent.VK_LEFT] == true)
			xPosition--;
		if (input.keys[KeyEvent.VK_RIGHT] == true)
			xPosition++;
		if (xPosition < 1) xPosition = 1;
		if (xPosition > world.width - 2) xPosition = world.width - 2;
		if (yPosition < 1) yPosition = 1;
		if (yPosition > world.height - 2) yPosition = world.height - 2;
	}
	
	public void draw(World world, int[] data) {
		data[(oldY + world.y) * TestGame.WIDTH + (oldX + world.x)] = 0;
		data[(yPosition + world.y) * TestGame.WIDTH + (xPosition + world.x)] = colorFlag;
		//TODO: Implement a way to draw a circle.
		//After implementing that, increase the radius every iteration.
		//This circle will be the player's attack wave.
		//TIP: http://www.cplusplus.com/forum/beginner/62634/
		
		for (int i = 0; i < this.xs.size(); i++) {
			circleDraw(data, xs.get(i), ys.get(i), rs.get(i), world.x, world.y);
		}
		if (isSelected) {
			this.selectionDraw(data, world.x, world.y);
		}
	}
	
	@Override
	public boolean isSelected() {
		return isSelected;
	}

	public void attack(Input input) {
		if (isSelected) {
			if (!attacking)
				attacking = input.keys[KeyEvent.VK_A];
			if (attacking) {
				attacking = false;
				this.setCircleDrawCenter(xPosition, yPosition);
				rs.add(0);
			}
		}
	}
	
	public void attackTick() {
		if (step % 4 == 0) {
			if (!rs.isEmpty()) {
				for (int i = 0; i < rs.size(); i++) {
					rs.set(i, rs.get(i) + 1);
					if (rs.get(i) > 5) {
						xs.remove(i);
						ys.remove(i);
						rs.remove(i);
					}
				}
			}
		}
	}
	
	public void select(World world, Input input) {
		if (input.leftMouseButton) {
			input.leftMouseButton = false;
			if (input.mouseX > this.xPosition - 2 + world.x && input.mouseX < this.xPosition + 2 + world.x && input.mouseY > this.yPosition - 2 + world.y && input.mouseY < this.yPosition + 2 + world.y) {
				isSelected = true;
			}
			else
				isSelected = false;

		}
	}
	
	public void move(World world, Input input, int diff) {
		if (step > diff) {
			step = 0;
			if (input.rightMouseButton) {
				input.rightMouseButton = false;
				if (isSelected) {
					newX = input.mouseX - world.x;
					newY = input.mouseY - world.y;
				}
			}
			if (newX == -1 && newY == -1)
				return;
			if (newX == this.xPosition && newY == this.yPosition)
				return;
			if (newX != this.xPosition && newY != this.yPosition) {
				dx = (newX - this.xPosition);
				dy = (newY - this.yPosition);
				double dd = Math.sqrt(dx * dx + dy * dy);
				dx /= dd;
				dy /= dd;
				this.xPosition += (int) Math.rint(dx);
				this.yPosition += (int) Math.rint(dy);
			}
			else {
				newX = this.xPosition;
				newY = this.yPosition;
			}
		}
		else
			step++;
	}
	
	public void tick(World world, Input input) {
		select(world, input);
		attack(input);
		attackTick();
		move(world, input, 10);
	}
	
	@Override
	public int getEntitySize() {
		return 1;
	}
	
	@Override
	public int getSelectCursorColor() {
		return 0xFF00AA00;
	}
}
