package main;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Random;

public class DeadGame {
	
	public InputHandler inputHandler;
	public ArrayList<Building> buildings;
	public ArrayList<Unit> units;
	
	private boolean switchColorFlag;
	private boolean gameOverFlag;

	public DeadGame(GameComponent g) {
		inputHandler = new InputHandler();
		buildings = new ArrayList<Building>();
		units = new ArrayList<Unit>();
		switchColorFlag = false;
		gameOverFlag = false;
		
		g.addMouseListener(inputHandler);
		g.addMouseMotionListener(inputHandler);
		
		initialize();
	}

	public void tick() {
		if (!gameOverFlag) {
			if (inputHandler.mouseClicked) {
				inputHandler.mouseClicked = false;
				boolean hasBuilding = false;
				for (Building b : buildings) {
					if (inputHandler.mouseX == b.x && inputHandler.mouseY == b.y) {
						hasBuilding = true;
						break;
					}
				}
				if (hasBuilding == false) {
					switchColorFlag = !switchColorFlag;
					Building b = new Building();
					b.x = inputHandler.mouseX;
					b.y = inputHandler.mouseY;
					if (switchColorFlag)
						b.setColor(0xFF00FF00);
					else
						b.setColor(0xFFFF0000);
					buildings.add(b);
				}
			}
			for (Iterator<Building> i = buildings.iterator(); i.hasNext();) {
				Building b = i.next();
				b.tick();
				if (b.isSendingUnitOut()) {
					Unit unit = new Unit(b.getColor());
					unit.setSpawnBuilding(b);
					//unit.findNearestBuilding(buildings);
					units.add(unit);
					b.finishedSendingUnitOut();
				}
				if (b.shouldDespawn())
					i.remove();
			}
			for (Iterator<Unit> i = units.iterator(); i.hasNext();) {
				Unit u = i.next();
				u.tick();
				u.findNearestEntity(buildings, units);
				//u.findNearestBuilding(buildings);
				if (u.shouldDespawn()) {
					i.remove();
					u.findNearestEntity(buildings, units);
				}
			}
			losingCondition();
		}
	}
	
	public void render(int[] data) {
		if (!gameOverFlag) {
			clear(data);
			renderBuildings(data);
			renderUnits(data);
			renderMouseCursor(data);
		}
		else {
			fade(data);
		}
	}
	
	private void renderMouseCursor(int[] data) {
		int x = inputHandler.mouseX / GameComponent.SCALE;
		int y = inputHandler.mouseY / GameComponent.SCALE;
		int length = y * (GameComponent.WIDTH / GameComponent.SCALE) + x;
		if (length < data.length && length >= 0)
			data[length] = -1;
	}
	
	private void renderBuildings(int[] data) {
		for (Building b : buildings) {
			b.render(data);
		}
	}
	
	private void renderUnits(int[] data) {
		for (Unit u : units)
			u.render(data);
	}

	private void initialize() {
		Random random = new Random();
		int green = random.nextInt(3) + 1;
		int red = random.nextInt(3) + 1;
		
		for (int g = 0; g < green; g++) {
			Building b = new Building();
			b.x = random.nextInt(GameComponent.WIDTH);
			b.y = random.nextInt(GameComponent.HEIGHT);
			boolean hasBuilding = false;
			for (Building c : buildings) {
				if (inputHandler.mouseX == b.x && inputHandler.mouseY == b.y) {
					hasBuilding = true;
					break;
				}
			}
			if (hasBuilding == false) {
				b.setColor(0xFF00FF00);
				buildings.add(b);
			}
			else
				g--;
		}
		for (int g = 0; g < red; g++) {
			Building b = new Building();
			b.x = random.nextInt(GameComponent.WIDTH);
			b.y = random.nextInt(GameComponent.HEIGHT);
			boolean hasBuilding = false;
			for (Building c : buildings) {
				if (inputHandler.mouseX == b.x && inputHandler.mouseY == b.y) {
					hasBuilding = true;
					break;
				}
			}
			if (hasBuilding == false) {
				b.setColor(0xFFFF0000);
				buildings.add(b);
			}
			else
				g--;
		}
	}
	
	private void losingCondition() {
		int red = 0;
		int green = 0;
		for (Building b : buildings) {
			if (b.getTeamColor() == 0xFFFF0000)
				red++;
			if (b.getTeamColor() == 0xFF00FF00)
				green++;
		}
		if (red == 0 || green == 0) {
			//Game over.
			gameOverFlag = true;
		}
	}
	
	private void fade(int[] data) {
		//		int value = 0;
		//		int temp = 0;
		int temp = 0;
		for (int i = 0; i < data.length; i++) {
			//			value = data[i] & (0xFF << 24);
			//			if (value < 0xFF)
			//				value++;
			//			temp = (data[i] & ~(0xFF << 24)) | (value << 24);
			//			value = data[i] & (0xFF << 16);
			//			if (value < 0xFF)
			//				value++;
			//			temp |= (data[i] & ~(0xFF << 16)) | (value << 16);
			//			value = data[i] & (0xFF << 8);
			//			if (value < 0xFF)
			//				value++;
			//			temp |= (data[i] & ~(0xFF << 8)) | (value << 8);
			//			value = data[i] & (0xFF);
			//			if (value < 0xFF)
			//				value++;
			//			temp |= (data[i] & ~(0xFF)) | (value);
			//			data[i] = temp;
			if (data[i] != -1) {
				temp = (data[i] & (0xFF << 24)) >> 24;
				if (temp < 0xFF)
					temp++;
				data[i] = (data[i] & ~(0xFF << 24)) | (temp << 24);
				
				temp = (data[i] & (0xFF << 16)) >> 16;
				if (temp < 0xFF)
					temp++;
				data[i] = (data[i] & ~(0xFF << 16)) | (temp << 16);
				
				temp = (data[i] & (0xFF << 8)) >> 8;
				if (temp < 0xFF)
					temp++;
				data[i] = (data[i] & ~(0xFF << 8)) | (temp << 8);
				
				temp = (data[i] & (0xFF));
				if (temp < 0xFF)
					temp++;
				data[i] = (data[i] & ~(0xFF)) | (temp);
			}
		}
		//		if (value > 0xFF)
		//			value = 0xFF;
	}
	
	public void clear(int[] data) {
		for (int i = 0; i < data.length; i++)
			data[i] = 0;
	}
}
