package main;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferStrategy;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferInt;

import javax.swing.JFrame;
import javax.swing.JPanel;

public class GameComponent extends Canvas implements Runnable {
	private static final long serialVersionUID = 1L;
	public static final double FRAMES_PER_SECOND = 60.0;
	public static final int WIDTH = 640;
	public static final int HEIGHT = 480;
	public static final int SCALE = 4;

	private Thread gameThread;
	protected boolean threadRunning;
	
	private BufferedImage image;
	private int[] pixels;
	private DeadGame game;

	public GameComponent() {
		threadRunning = false;
		
		Dimension d = new Dimension(WIDTH, HEIGHT);
		this.setPreferredSize(d);
		this.setMaximumSize(d);
		this.setMinimumSize(d);
		this.setSize(d);
		
		image = new BufferedImage(WIDTH / SCALE, HEIGHT / SCALE, BufferedImage.TYPE_INT_ARGB);
		pixels = ((DataBufferInt) image.getRaster().getDataBuffer()).getData();
		
		game = new DeadGame(this);
	}
	
	@Override
	public void run() {
		long now = System.nanoTime();
		long lastTime = now;
		double unprocessedTime = 0.0;
		final double ticksPerSecond = 1000000000.0 / FRAMES_PER_SECOND;
		while (threadRunning) {
			now = System.nanoTime();
			unprocessedTime += (now - lastTime) / ticksPerSecond;
			lastTime = now;
			
			boolean render = false;
			while (unprocessedTime >= 1) {
				unprocessedTime -= 1;
				tick();
				render = true;
			}
			if (render)
				render();
			try {
				Thread.sleep(1);
			}
			catch (InterruptedException e) {
			}
		}
	}
	
	public synchronized void start() {
		if (!threadRunning) {
			threadRunning = true;
			gameThread = new Thread(this);
			gameThread.start();
		}
	}
	
	public synchronized void stop() {
		if (threadRunning) {
			threadRunning = false;
		}
	}
	
	public void tick() {
		game.tick();
	}
	
	public void render() {
		BufferStrategy bs = this.getBufferStrategy();
		if (bs == null) {
			this.createBufferStrategy(3);
			bs = this.getBufferStrategy();
		}
		
		int w = this.getWidth();
		int h = this.getHeight();
		
		Graphics g = bs.getDrawGraphics();
		
		g.setColor(new Color(24, 100, 150));
		g.fillRect(0, 0, w, h);
		
		game.render(pixels);
		
		g.drawImage(image, 0, 0, w, h, null);
		g.dispose();
		bs.show();
	}
	
	public static void main(String[] args) {
		GameComponent g = new GameComponent();
		JFrame frame = new JFrame("DeadGame");
		frame.add(new JPanel().add(g));
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setLocationRelativeTo(null);
		frame.setResizable(false);
		frame.pack();
		frame.setVisible(true);

		g.start();
	}

}
