package main;

import game.Mob;
import game.Player;
import game.World;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferStrategy;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferInt;

public class TestGame extends Canvas implements Runnable {

	private static final long serialVersionUID = 2L;
	public static final int SCALE = 10;
	public static int WIDTH = 0;
	public static int HEIGHT = 0;
	
	private boolean running = false;
	private BufferedImage image;
	private int[] pixels;
	private Input input;
	
	public World world;
	public Mob test;
	public Player player;
	
	public TestGame(int w, int h) {
		this.WIDTH = w;
		this.HEIGHT = h;

		Dimension size = new Dimension(w * SCALE, h * SCALE);
		this.setSize(size);
		this.setPreferredSize(size);
		this.setMaximumSize(size);
		this.setMinimumSize(size);
		
		input = new Input();
		this.addKeyListener(input);
		this.addMouseListener(input);

		image = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
		pixels = ((DataBufferInt) image.getRaster().getDataBuffer()).getData();
		
		world = new World(2, 7, 30, 20);
		test = new Mob(4, 5);
		player = new Player(12, 8);
		
		world.setMob(pixels, test);
		world.setPlayer(pixels, player);

	}
	
	public void tick(TestGame game) {
		player.tick(world, input);
		//test.attack(game, player, 10);
	}
	
	public void render(TestGame game) {
		BufferStrategy bs = this.getBufferStrategy();
		if (bs == null) {
			this.createBufferStrategy(3);
			return;
		}
		int w = this.getWidth();
		int h = this.getHeight();
		Graphics g = bs.getDrawGraphics();
		g.setColor(Color.BLACK);
		g.fillRect(0, 0, w, h);

		draw(game);

		g.drawImage(image, 0, 0, w, h, null);
		g.dispose();
		bs.show();
	}
	
	public void clear() {
		for (int a = 0; a < pixels.length; a++)
			pixels[a] = 0;
	}

	public void draw(TestGame game) {
		clear();
		world.clearWorld(game, pixels);
		player.draw(world, pixels);
		test.draw(world, pixels);
	}

	@Override
	public void run() {
		long now = System.nanoTime();
		long lastTime = now;
		double unprocessed = 0.0;
		this.requestFocus();
		while (running) {
			now = System.nanoTime();
			unprocessed += (now - lastTime) / (1000000000.0 / 60.0);
			lastTime = now;
			
			boolean r = false;
			while (unprocessed >= 1) {
				unprocessed -= 1;
				tick(this);
				r = true;
			}
			if (r)
				render(this);
			
			try {
				Thread.sleep(1);
			}
			catch (InterruptedException e) {
			}
		}
	}
	
	public synchronized void start() {
		if (!running) {
			running = true;
			new Thread(this).start();
		}
	}
	
	public synchronized void stop() {
		if (running)
			running = false;
	}
}
