package main;

import java.util.ArrayList;

public class Unit extends Location implements Entity {
	public int color;
	public int teamColor;
	public Building target;
	public Building start;
	public Location targetEntity;
	private boolean shouldDespawn;
	
	public Unit(int c) {
		teamColor = c;
		color = c;
		targetEntity = null;
	}

	public Building findNearestBuilding(ArrayList<Building> list) {
		Building temp = null;
		long minDistance = GameComponent.WIDTH * GameComponent.WIDTH + GameComponent.HEIGHT * GameComponent.HEIGHT;
		for (Building b : list) {
			if (start != null) {
				if (b == start)
					continue;
			}
			if (b.getTeamColor() == this.teamColor)
				continue;
			long distanceSquared = (start.x - b.x) * (start.x - b.x) + (start.y - b.y) * (start.y - b.y);
			if (minDistance > distanceSquared) {
				if (b.getTeamColor() != this.teamColor) {
					temp = b;
					minDistance = distanceSquared;
				}
			}
		}
		target = temp;
		return target;
	}
	
	public Unit findNearestUnit(ArrayList<Unit> list) {
		Unit temp = null;
		long minDistance = GameComponent.WIDTH * GameComponent.WIDTH + GameComponent.HEIGHT * GameComponent.HEIGHT;
		for (Unit u : list) {
			if (this.x == u.x && this.y == u.y)
				continue;
			if (u.getTeamColor() == this.teamColor)
				continue;
			long distanceSquared = (u.x - this.x) * (u.x - this.x) + (u.y - this.y) * (u.y - this.y);
			if (minDistance > distanceSquared) {
				if (u.getTeamColor() != this.teamColor) {
					temp = u;
					minDistance = distanceSquared;
				}
			}
		}
		return temp;
	}
	
	public void findNearestEntity(ArrayList<Building> buildings, ArrayList<Unit> units) {
		Building b = this.findNearestBuilding(buildings);
		Unit u = this.findNearestUnit(units);
		
		if (shouldDespawn) {
			if (b != null && u == null) {
				b.takeDamage(this);
			}
		}

		if (b == null && u == null) {
			targetEntity = null;
			return;
		}
		//Priority - HIGH:      Units first.
		if (b == null && u != null) {
			targetEntity = u;
			return;
		}
		//Priority - LOW:       Buildings last.
		if (b != null && u == null) {
			targetEntity = b;
			return;
		}

		long buildingDist = (b.x - this.x) * (b.x - this.x) + (b.y - this.y) * (b.y - this.y);
		long unitDist = (u.x - this.x) * (u.x - this.x) + (u.y - this.y) * (u.y - this.y);
		targetEntity = buildingDist < unitDist ? b : u;
	}

	public void setSpawnBuilding(Building b) {
		start = b;
		this.x = b.x;
		this.y = b.y;
		shouldDespawn = false;
	}
	
	public void setTargetBuilding(Building b) {
		target = b;
	}
	
	public void tick() {
		move();
		despawn();
	}
	
	public void render(int[] data) {
		int x = this.x / GameComponent.SCALE;
		int y = this.y / GameComponent.SCALE;
		int length = y * (GameComponent.WIDTH / GameComponent.SCALE) + x;
		if (length < data.length && length >= 0)
			data[length] = color;
	}
	
	private void move() {
		if (targetEntity == null || start == null)
			return;
		else {
			double dx = targetEntity.x - this.x;
			double dy = targetEntity.y - this.y;
			
			double dist = Math.sqrt(dx * dx + dy * dy);
			//if (dist > 0) {
			dx /= dist;
			dy /= dist;
			//}
			this.x += (int) (Math.rint(dx));
			this.y += (int) (Math.rint(dy));
		}
	}
	
	public boolean shouldDespawn() {
		return shouldDespawn;
	}
	
	private void despawn() {
		if (targetEntity == null || start == null)
			return;
		if (this.x == targetEntity.x && this.y == targetEntity.y)
			shouldDespawn = true;
	}
	
	public int getColor() {
		return color;
	}
	
	public int getTeamColor() {
		return teamColor;
	}
}
